<?php

namespace CorporateTrainingBundle\Biz\Course\Service\Impl;

use AppBundle\Common\ArrayToolkit;
use Biz\Course\Service\CourseService;
use Biz\Course\Service\Impl\CourseSetServiceImpl as BaseCourseSetServiceImpl;
use Codeages\Biz\Framework\Event\Event;

class CourseSetServiceImpl extends BaseCourseSetServiceImpl
{
    public function createCourseSet($courseSet)
    {
        if (!$this->hasCourseSetManageRole()) {
            throw $this->createAccessDeniedException('You have no access to Course Set Management');
        }

        $created = $this->addCourseSet($courseSet);

        // 同时创建默认的教学计划
        // XXX
        // 1. 是否创建默认教学计划应该是可配的；
        // 2. 教学计划的内容（主要是学习模式、有效期模式）也应该是可配的
        //内训只需要一个教学计划，默认不创建
        // $defaultCourse = $this->generateDefaultCourse($created);
        // $this->getCourseService()->createCourse($defaultCourse);
        $course = $this->addCourse($created, $courseSet);
        $this->getCourseSetDao()->update($created['id'], array('defaultCourseId' => $course['id']));
        $this->getLogService()->info('course', 'create', sprintf('创建课程《%s》(#%s)', $created['title'], $created['id']));

        return $created;
    }

    public function updateCourseSet($id, $fields)
    {
        if (!ArrayToolkit::requireds($fields, array('title', 'categoryId', 'serializeMode'))) {
            throw $this->createInvalidArgumentException('Lack of required fields');
        }
        if (!in_array($fields['serializeMode'], array('none', 'serialized', 'finished'))) {
            throw $this->createInvalidArgumentException('Invalid Param: serializeMode');
        }

        $courseSet = $this->tryManageCourseSet($id);

        $fields = ArrayToolkit::parts(
            $fields,
            array(
                'title',
                'subtitle',
                'tags',
                'categoryId',
                'serializeMode',
                // 'summary',
                'smallPicture',
                'middlePicture',
                'largePicture',
                'teacherIds',
                'orgCode',
                'orgId',
            )
        );

        if ($fields['tags'] !== null) {
            $tags = explode(',', $fields['tags']);
            $tags = $this->getTagService()->findTagsByNames($tags);
            $tagIds = ArrayToolkit::column($tags, 'id');
            $fields['tags'] = $tagIds;
        }

        $fields = $this->filterFields($fields);

        if (isset($fields['summary'])) {
            $fields['summary'] = $this->purifyHtml($fields['summary'], true);
        }

        $this->updateCourseSerializeMode($courseSet, $fields);
        if (empty($fields['subtitle'])) {
            $fields['subtitle'] = null;
        }

        $courseSet = $this->getCourseSetDao()->update($courseSet['id'], $fields);

        $this->getLogService()->info('course', 'update', "修改课程《{$courseSet['title']}》(#{$courseSet['id']})");
        $this->dispatchEvent('course-set.update', new Event($courseSet));

        return $courseSet;
    }

    public function tryManageCourseSet($id)
    {
        $user = $this->getCurrentUser();
        if (!$user->isLogin()) {
            throw $this->createAccessDeniedException('user not login');
        }

        $courseSet = $this->getCourseSetDao()->get($id);

        if (empty($courseSet)) {
            throw $this->createNotFoundException("CourseSet#{$id} Not Found");
        }

        if (!in_array($courseSet['orgId'], $user['accessibleOrgIds'])) {
            throw $this->createAccessDeniedException("org#{$courseSet['orgId']} is not accessible");
        }

        if ($courseSet['parentId'] > 0) {
            $classroomCourse = $this->getClassroomService()->getClassroomCourseByCourseSetId($id);
            if (!empty($classroomCourse)) {
                $classroom = $this->getClassroomService()->getClassroom($classroomCourse['classroomId']);
                if (!empty($classroom) && $classroom['headTeacherId'] == $user['id']) {
                    //班主任有权管理班级下所有课程
                    return $courseSet;
                }
            }
        }
        if (!$this->hasCourseSetManageRole($id)) {
            throw $this->createAccessDeniedException('can not access');
        }

        return $courseSet;
    }

    protected function addCourseSet($courseSet)
    {
        if (!ArrayToolkit::requireds($courseSet, array('title', 'type'))) {
            throw $this->createInvalidArgumentException('Lack of required fields');
        }

        if (!in_array($courseSet['type'], static::courseSetTypes())) {
            throw $this->createInvalidArgumentException('Invalid Param: type');
        }

        $courseSet = ArrayToolkit::parts(
            $courseSet,
            array(
                'type',
                'title',
                'orgCode',
                'orgId',
            )
        );

        $courseSet['status'] = 'draft';
        $courseSet['creator'] = $this->getCurrentUser()->getId();
        $created = $this->getCourseSetDao()->create($courseSet);

        return $created;
    }

    protected function addCourse($courseSet, $created)
    {
        $created = array_merge($created, $courseSet);
        $course = $this->generateCourse($created);

        return $this->getCourseService()->createCourse($course);
    }

    protected function generateCourse($created)
    {
        $created = $this->prepareExpiryMode($created);
        $course = array(
            'courseSetId' => $created['id'],
            'title' => $created['title'],
            'expiryMode' => $created['expiryMode'],
            'learnMode' => empty($created['learnMode']) ? CourseService::FREE_LEARN_MODE : $created['learnMode'],
            'courseType' => empty($created['courseType']) ? CourseService::DEFAULT_COURSE_TYPE : $created['courseType'],
            'isDefault' => 1,
            'isFree' => 1,
            'serializeMode' => $created['serializeMode'],
            'status' => 'draft',
            'type' => $created['type'],
            'expiryEndDate' => $created['expiryEndDate'],
            'expiryStartDate' => $created['expiryStartDate'],
            'expiryDays' => $created['expiryDays'],
        );

        return $course;
    }

    private function prepareExpiryMode($data)
    {
        if (empty($data['expiryMode']) || $data['expiryMode'] != 'days') {
            unset($data['deadlineType']);
        }
        if (!empty($data['deadlineType'])) {
            if ($data['deadlineType'] == 'end_date') {
                $data['expiryMode'] = 'end_date';
                $data['expiryEndDate'] = $data['deadline'];

                return $data;
            } else {
                $data['expiryMode'] = 'days';

                return $data;
            }
        }

        return $data;
    }

    public function findCourseSetsByCurrentOrgAndFullSiteOrg($conditions, $orderBy, $start, $limit)
    {
        $currentOrgCourseSets = $this->searchCourseSets($conditions, $orderBy, $start, $limit);
        unset($conditions['likeOrgCode']);
        $conditions['orgCode'] = '1.';
        $fullSiteOrgCourseSets = $this->searchCourseSets($conditions, $orderBy, $start, $limit);
        $courseSets = array_merge($currentOrgCourseSets, $fullSiteOrgCourseSets);

        return $courseSets;
    }

    public function initOrgsRelation()
    {
        $fields = array('1', '1.');

        return $this->getCourseSetDao()->initOrgsRelation($fields);
    }
}
