<?php

namespace CorporateTrainingBundle\Biz\DataStatistics\Event;

use Codeages\Biz\Framework\Event\Event;
use Codeages\PluginBundle\Event\EventSubscriber;
use CorporateTrainingBundle\Biz\DataStatistics\EventProcessor\EventProcessorFactory;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class DataStatisticsEventSubscriber extends EventSubscriber implements EventSubscriberInterface
{
    public static function getSubscribedEvents()
    {
        return array(
            'course.join' => 'onCourseJoin',
            'course.task.finish' => 'onTaskFinish',
            'course.view' => 'onCourseView',
            'task.view' => 'onTaskView',
            'wave.learn.time' => 'onWaveLearnTime',
            'user.post.update' => 'onUserPostUpdate',
        );
    }

    public function onCourseJoin(Event $event)
    {
        $course = $event->getSubject();
        $userId = $event->getArgument('userId');

        $user = $this->getUserService()->getUser($userId);

        if (empty($user) || empty($course) || !empty($user['locked'])) {
            return;
        }

        if (in_array($user['id'], $course['teacherIds'])) {
            return;
        }

        $data = array(
            'courseId' => $course['id'],
            'categoryId' => $course['categoryId'],
            'userId' => $user['id'],
            'postId' => $user['postId'],
        );

        $data = $this->processClassroomCourseData($course, $data);

        $this->processDailyRecord('course.join', $data);
    }

    public function onTaskFinish(Event $event)
    {
        $taskResult = $event->getSubject();
        $user = $event->getArgument('user');

        $course = $this->getCourseService()->getCourse($taskResult['courseId']);

        if (empty($user) || empty($course) || !empty($user['locked'])) {
            return;
        }

        if (in_array($user['id'], $course['teacherIds'])) {
            return;
        }

        $data = array(
            'courseId' => $course['id'],
            'categoryId' => $course['categoryId'],
            'userId' => $user['id'],
            'postId' => $user['postId'],
        );

        if ($course['serializeMode'] != 'serialized') {
            $learnedCompulsoryTaskNum = $this->getTaskResultService()->countFinishedCompulsoryTasksByUserIdAndCourseId($user['id'], $course['id']);

            if ($learnedCompulsoryTaskNum >= $course['compulsoryTaskNum']) {
                $data['courseStatus'] = 1;
            }
        }

        $data = $this->processClassroomCourseData($course, $data);

        $this->processDailyRecord('task.finish', $data);
    }

    public function onCourseView(Event $event)
    {
        $course = $event->getSubject();
        $userId = $event->getArgument('userId');

        $user = $this->getUserService()->getUser($userId);

        if (empty($user) || empty($course) || !empty($user['locked'])) {
            return;
        }

        if (in_array($user['id'], $course['teacherIds'])) {
            return;
        }
        $data = array(
            'courseId' => $course['id'],
            'categoryId' => $course['categoryId'],
            'userId' => $user['id'],
            'postId' => $user['postId'],
        );

        $data = $this->processClassroomCourseData($course, $data);

        $this->processDailyRecord('course.view', $data);
    }

    public function onTaskView(Event $event)
    {
        $courseMember = $event->getSubject();

        $user = $this->getUserService()->getUser($courseMember['userId']);
        $course = $this->getCourseService()->getCourse($courseMember['courseId']);

        if (empty($courseMember) || empty($user) || empty($course) || !empty($user['locked'])) {
            return;
        }

        if (in_array($user['id'], $course['teacherIds'])) {
            return;
        }

        $data = array(
            'courseId' => $course['id'],
            'categoryId' => $course['categoryId'],
            'userId' => $user['id'],
            'postId' => $user['postId'],
        );

        $data = $this->processClassroomCourseData($course, $data);

        $this->processDailyRecord('task.view', $data);
    }

    public function onWaveLearnTime(Event $event)
    {
        $taskResultId = $event->getSubject();
        $learnTime = $event->getArgument('learnTime');

        $taskResult = $this->getTaskResultService()->getTaskResultById($taskResultId);

        $user = $this->getUserService()->getUser($taskResult['userId']);

        $courseId = $taskResult['courseId'];
        $course = $this->getCourseService()->getCourse($courseId);

        if (empty($user) || empty($course) || !empty($user['locked'])) {
            return;
        }

        if (in_array($user['id'], $course['teacherIds'])) {
            return;
        }

        $data = array(
            'courseId' => $course['id'],
            'categoryId' => $course['categoryId'],
            'userId' => $user['id'],
            'postId' => $user['postId'],
            'learnTime' => $learnTime,
        );

        $data = $this->processClassroomCourseData($course, $data);

        $this->processDailyRecord('wave.learn.time', $data);
    }

    public function onUserPostUpdate(Event $event)
    {
        $user = $event->getSubject();
        $userReport = $this->getUserReportService()->getUserLearnReportByUserId($user['id']);
        if (empty($userReport)) {
            return;
        }
        $this->getUserReportService()->updateLearnReport($userReport['id'], array('postId' => $user['postId']));
    }

    protected function processClassroomCourseData($course, $data)
    {
        if (!empty($course['parentId'])) {
            $classroom = $this->getClassroomService()->getClassroomByCourseId($course['id']);
            if (!empty($classroom)) {
                $data['classroomId'] = $classroom['id'];
            }
        }

        return $data;
    }

    protected function processDailyRecord($type, $data)
    {
        return EventProcessorFactory::create($type)->process($data);
    }

    /**
     * @return \Biz\User\Service\Impl\UserServiceImpl
     */
    protected function getUserService()
    {
        return $this->createService('User:UserService');
    }

    /**
     * @return \Biz\Course\Service\Impl\CourseServiceImpl
     */
    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }

    /**
     * @return \Biz\Task\Service\Impl\TaskResultServiceImpl
     */
    protected function getTaskResultService()
    {
        return $this->createService('Task:TaskResultService');
    }

    /**
     * @return \Biz\Classroom\Service\Impl\ClassroomServiceImpl
     */
    protected function getClassroomService()
    {
        return $this->createService('Classroom:ClassroomService');
    }

    /**
     * @return \CorporateTrainingBundle\Biz\DataStatistics\Service\Impl\UserLearnReportServiceImpl
     */
    protected function getUserReportService()
    {
        return $this->createService('CorporateTrainingBundle:DataStatistics:UserLearnReportService');
    }

    protected function createService($alias)
    {
        return $this->getBiz()->service($alias);
    }
}
