<?php

namespace CorporateTrainingBundle\Biz\Exporter;

use AppBundle\Common\ArrayToolkit;
use CorporateTrainingBundle\Common\OrgToolkit;
use Biz\Org\Service\OrgService;
use Biz\User\Service\UserService;
use CorporateTrainingBundle\Biz\Post\Service\PostService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentMemberService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentService;

class TrainingAssignmentMemberExporter extends AbstractExporter
{
    public function canExport($parameters)
    {
        return $this->getTrainingAssignmentService()->canManageTrainingAssignment($parameters['assignmentId']);
    }

    public function getExportFileName()
    {
        return 'training_assignment_member.xls';
    }

    public function getSortedHeadingRow()
    {
        return array(
            array('code' => 'truename', 'title' => '姓名'),
            array('code' => 'nickname', 'title' => '用户名'),
            array('code' => 'org', 'title' => '部门'),
            array('code' => 'post', 'title' => '岗位'),
            array('code' => 'totalLearnTime', 'title' => '学习时长'),
            array('code' => 'status', 'title' => '完成情况'),
        );
    }

    public function buildExportData($parameters)
    {
        $trainingAssignment = $this->getTrainingAssignmentService()->getTrainingAssignment($parameters['assignmentId']);
        $conditions = $this->prepareMemberListSearchConditions($parameters, $trainingAssignment);
        $members = $this->getTrainingAssignmentMemberService()->searchTrainingAssignmentMembers(
            $conditions,
            array('id' => 'DESC'),
            0,
            PHP_INT_MAX
        );

        $userIds = ArrayToolkit::column($members, 'userId');
        $userProfiles = $this->getUserService()->findUserProfilesByIds($userIds);
        $userProfiles = ArrayToolkit::index($userProfiles, 'id');
        $users = $this->getUserService()->findUsersByIds($userIds);
        $users = ArrayToolkit::index($users, 'id');

        $orgIds = array();

        foreach ($users as $user) {
            $orgIds = array_merge($orgIds, $user['orgIds']);
        }
        $orgIds = array_values(array_unique($orgIds));
        $orgs = $this->getOrgService()->findOrgsByIds($orgIds);
        $orgs = ArrayToolkit::index($orgs, 'id');

        $posts = $this->getPostService()->findPostsByIds(ArrayToolkit::column($users, 'postId'));
        $posts = ArrayToolkit::index($posts, 'id');

        $usersTotalLearnTime = $this->createAssignmentStrategy()->calculateMembersTotalLearnTime(
            $trainingAssignment,
            $userIds
        );

        $memberData = array();
        foreach ($members as $member) {
            $memberData[] = array(
                $user = $users[$member['userId']],
                'truename' => empty($userProfiles[$member['userId']]) ? '-' : $userProfiles[$member['userId']]['truename'],
                'nickname' => empty($users[$member['userId']]) ? '-' : $users[$member['userId']]['nickname'],
                'org' => OrgToolkit::buildOrgsNames($user['orgIds'], $orgs),
                'post' => empty($posts[$users[$member['userId']]['postId']]) ? '-' : $posts[$users[$member['userId']]['postId']]['name'],
                'totalLearnTime' => empty($usersTotalLearnTime[$member['userId']]) ? 0 : $usersTotalLearnTime[$member['userId']],
                'status' => ($member['status'] == 'finished') ? '已完成' : '进行中',
            );
        }

        $exportData[] = array(
            'data' => $memberData,
        );

        return $exportData;
    }

    protected function prepareMemberListSearchConditions($conditions, $trainingAssignment)
    {
        if (!empty($conditions['orgIds'])) {
            $conditions['orgIds'] = empty($orgIds) ? explode(',', $conditions['orgIds']) : $orgIds;

            $orgUserIds = $this->getAssignmentOrgUserIds($conditions['assignmentId'], $conditions['orgIds']);
            $conditions['userIds'] = $orgUserIds;
        }

        if (!empty($conditions['username'])) {
            $userIds = $this->getUserIdsByNickNameOrTrueName($conditions['username']);
            if (empty($conditions['userIds']) || empty($userIds)) {
                $conditions['userIds'] = array();
            } else {
                $conditions['userIds'] = array_intersect($conditions['userIds'], $userIds);
            }
            unset($conditions['username']);
        }

        $conditions['userIds'] = empty($conditions['userIds']) ? array(-1) : $conditions['userIds'];

        if (!empty($conditions['status']) && $conditions['status'] == 'all') {
            $conditions['statuses'] = array('notStart', 'ongoing', 'finished');
            unset($conditions['status']);
        }

        if (!empty($conditions['status']) && $conditions['status'] == 'unfinished') {
            $conditions['statuses'] = array('notStart', 'ongoing');
            unset($conditions['status']);
        }

        if (!empty($conditions['showDelay']) && !empty($trainingAssignment['startTime']) && !empty($trainingAssignment['endTime'])) {
            $conditions['finishedTime_GT'] = $trainingAssignment['endTime'];
            unset($conditions['showDelay']);
        }

        return $conditions;
    }

    protected function getAssignmentOrgUserIds($assignmentId, $orgIds)
    {
        $members = $this->getTrainingAssignmentMemberService()->findMembersByAssignmentId($assignmentId);

        $userIds = ArrayToolkit::column($members, 'userId');

        $users = $this->getUserOrgService()->searchUserOrgs(
            array('orgIds' => $orgIds, 'userIds' => $userIds),
            array(),
            0,
            PHP_INT_MAX
        );

        return ArrayToolkit::column($users, 'userId');
    }

    protected function getUserIdsByNickNameOrTrueName($name)
    {
        $users = $this->getUserService()->searchUsers(
            array('nickname' => $name),
            array('id' => 'DESC'),
            0,
            20
        );

        $userIds = ArrayToolkit::column($users, 'id');

        $users = $this->getUserService()->searchUserProfiles(
            array('truename' => $name),
            array('id' => 'DESC'),
            0,
            20
        );

        $userIds = array_merge($userIds, ArrayToolkit::column($users, 'id'));

        return $userIds;
    }

    protected function createAssignmentStrategy()
    {
        return $this->biz->offsetGet('assignment_strategy_context')->createStrategy();
    }

    /**
     * @return TrainingAssignmentService
     */
    protected function getTrainingAssignmentService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentService');
    }

    /**
     * @return TrainingAssignmentMemberService
     */
    protected function getTrainingAssignmentMemberService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentMemberService');
    }

    /**
     * @return UserService
     */
    protected function getUserService()
    {
        return $this->createService('User:UserService');
    }

    /**
     * @return OrgService
     */
    protected function getOrgService()
    {
        return $this->createService('Org:OrgService');
    }

    /**
     * @return PostService
     */
    protected function getPostService()
    {
        return $this->createService('CorporateTrainingBundle:Post:PostService');
    }

    protected function getUserOrgService()
    {
        return $this->createService('CorporateTrainingBundle:User:UserOrgService');
    }
}
