<?php

namespace CorporateTrainingBundle\Biz\OfflineCourse\Service\Impl;

use AppBundle\Common\ArrayToolkit;
use Biz\BaseService;
use Biz\System\Service\LogService;
use Biz\User\Service\UserService;
use Codeages\Biz\Framework\Event\Event;
use CorporateTrainingBundle\Biz\OfflineCourse\Dao\MemberDao;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\EnrollmentRecordService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\MemberService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\OfflineCourseService;

class MemberServiceImpl extends BaseService implements MemberService
{
    public function createMember($member)
    {
        if (!ArrayToolkit::requireds($member, array('offlineCourseId', 'userId'))) {
            throw $this->createServiceException('parameter is invalid!');
        }

        $member = $this->filterFields($member);

        $member = $this->getMemberDao()->create($member);
        $this->dispatchEvent(
            'offline.become.member',
            new Event($member)
        );

        return $member;
    }

    public function updateMember($id, $fields)
    {
        $fields = $this->filterFields($fields);

        return $this->getMemberDao()->update($id, $fields);
    }

    public function deleteMember($id)
    {
        $member = $this->getMember($id);
        if (empty($member)) {
            throw $this->createNotFoundException("member #{$id} not found");
        }

        $result = $this->getMemberDao()->delete($id);
        $this->dispatchEvent(
            'offline.member.delete',
            new Event($member)
        );

        return $result;
    }

    public function getMember($memberId)
    {
        return $this->getMemberDao()->get($memberId);
    }

    public function countMembers($conditions)
    {
        return $this->getMemberDao()->count($conditions);
    }

    public function searchMembers(array $conditions, array $orderBys, $start, $limit)
    {
        return $this->getMemberDao()->search($conditions, $orderBys, $start, $limit);
    }

    public function findDistinctUserIdsByDate($startDate, $endDate)
    {
        $endTime = strtotime(date('Y-m-d 23:59:59', $endDate));

        return $this->getMemberDao()->findDistinctUserIdsByCreatedTime($startDate, $endTime);
    }

    protected function filterFields($fields)
    {
        return ArrayToolkit::parts(
            $fields,
            array(
                'userId',
                'offlineCourseId',
                'attendedStatus',
                'score',
                'passedStatus',
                'evaluate',
                'joinStatus',
                'joinType',
            )
        );
    }

    public function findMembersByOfflineCourseId($courseId)
    {
        return $this->getMemberDao()->findByCourseId($courseId);
    }

    public function getMemberByCourseIdAndUserId($courseId, $userId)
    {
        return $this->getMemberDao()->getByCourseIdAndUserId($courseId, $userId);
    }

    public function becomeMember($courseId, $userId)
    {
        $canBecomeMember = $this->canBecomeMember($courseId, $userId);

        if (!$canBecomeMember) {
            throw $this->createServiceException('can not become member');
        }

        $member = $this->getMemberByCourseIdAndUserId($courseId, $userId);
        if (empty($member)) {
            $fields['joinStatus'] = 'join';
            $fields['offlineCourseId'] = $courseId;
            $fields['userId'] = $userId;

            $member = $this->createMember($fields);
        } else {
            $member = $this->updateMember($member['id'], array('joinStatus' => 'join'));
        }

        $this->getCourseService()->refreshMemberCount($courseId);

        return $member;
    }

    public function becomeMemberByImport($courseId, $userId)
    {
        if (!$this->getCourseService()->hasCourseManageRole()) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        $course = $this->getCourseService()->getOfflineCourse($courseId);
        if (empty($course)) {
            throw $this->createNotFoundException("Course #{$courseId} not found");
        }

        if ('published' != $course['status']) {
            throw $this->createServiceException("Course #{$courseId} not publish");
        }

        $user = $this->getUserService()->getUser($userId);

        if (empty($user)) {
            throw  $this->createNotFoundException("User #{$userId} not found");
        }

        $member = $this->getMemberByCourseIdAndUserId($courseId, $userId);
        if ($member) {
            throw  $this->createServiceException("Member #{$member['id']} has exist");
        }

        $record = $this->getEnrollmentRecordService()->getLatestEnrollmentRecordByCourseIdAndUserId($courseId, $userId);
        if (!empty($record) && 'submitted' == $record['status']) {
            $this->getEnrollmentRecordService()->updateEnrollmentRecord($record['id'], array('status' => 'approved'));
        }

        $fields['joinStatus'] = 'join';
        $fields['offlineCourseId'] = $courseId;
        $fields['userId'] = $userId;
        $fields['joinType'] = 'import';

        $this->getLogService()->info(
            'offlineCourse',
            'import_user',
            "import user {$userId} into course {$courseId}"
        );

        $result = $this->createMember($fields);

        $this->getCourseService()->refreshMemberCount($courseId);

        return $result;
    }

    public function isMember($courseId, $userId)
    {
        $user = $this->getUserService()->getUser($userId);
        if (empty($user)) {
            return false;
        }

        $course = $this->getCourseService()->getOfflineCourse($courseId);
        if (empty($course)) {
            return false;
        }

        $member = $this->getMemberByCourseIdAndUserId($courseId, $userId);

        if (empty($member) || ('join' != $member['joinStatus'])) {
            return false;
        }

        return true;
    }

    public function attendMember($id, $attendedStatus)
    {
        $this->checkMemberExist($id);

        return $this->updateMember($id, array('attendedStatus' => $attendedStatus));
    }

    public function signIn($id)
    {
        return $this->attendMember($id, 'attended');
    }

    public function gradeMember($id, $fields)
    {
        $this->checkMemberExist($id);

        $fields = ArrayToolkit::parts($fields, array(
            'score',
            'passedStatus',
            'evaluate',
        ));

        if ($fields['score'] > 100 || $fields['score'] < 0) {
            throw $this->createServiceException('The score must between 0-100');
        }

        return $this->updateMember($id, $fields);
    }

    public function removeMember($id)
    {
        $member = $this->getMember($id);

        if (empty($member)) {
            throw $this->createNotFoundException("member #{$id} not found");
        }

        if (!$this->getCourseService()->hasCourseManageRole()) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        $result = $this->getMemberDao()->update($id, array('joinStatus' => 'removed'));

        $this->getCourseService()->refreshMemberCount($member['offlineCourseId']);

        return $result;
    }

    public function statisticMembersAttendStatusByCourseId($courseId)
    {
        return $this->getMemberDao()->statisticAttendStatusByCourseId($courseId);
    }

    public function statisticMemberPassStatusByCourseId($courseId)
    {
        return $this->getMemberDao()->statisticPassStatusByCourseId($courseId);
    }

    public function statisticMemberScoreByCourseId($courseId)
    {
        return $this->getMemberDao()->statisticScoreByCourseId($courseId);
    }

    protected function checkMemberExist($id)
    {
        $member = $this->getMember($id);

        if (empty($member)) {
            throw $this->createNotFoundException("member #{$id} not found");
        }
    }

    protected function canBecomeMember($courseId, $userId)
    {
        $course = $this->getCourseService()->getOfflineCourse($courseId);

        if (empty($course)) {
            return false;
        }

        if (!in_array($course['status'], array('published'))) {
            return false;
        }

        $user = $this->getUserService()->getUser($userId);

        if (empty($user)) {
            return false;
        }

        $currentUser = $this->getCurrentUser();
        if ($currentUser->hasPermission('admin_offline_course_manage')) {
            return true;
        }

        if (time() > $course['enrollmentEndDate']) {
            return false;
        }

        if (!empty($course['maxStudentNum']) && $course['studentNum'] >= $course['maxStudentNum']) {
            return false;
        }

        $member = $this->getMemberByCourseIdAndUserId($courseId, $userId);

        if (!empty($course['requireAudit'])) {
            $latestEnrollmentRecord = $this->getEnrollmentRecordService()->getLatestEnrollmentRecordByCourseIdAndUserId(
                $courseId,
                $userId
            );

            if (empty($latestEnrollmentRecord || 'approved' != $latestEnrollmentRecord['status'])) {
                return false;
            }
        }

        if (empty($member) || 'removed' === $member['joinStatus']) {
            return true;
        }

        return false;
    }

    /**
     * @return OfflineCourseService
     */
    protected function getCourseService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:OfflineCourseService');
    }

    /**
     * @return UserService
     */
    protected function getUserService()
    {
        return $this->createService('User:UserService');
    }

    /**
     * @return EnrollmentRecordService
     */
    protected function getEnrollmentRecordService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:EnrollmentRecordService');
    }

    /**
     * @return LogService
     */
    protected function getLogService()
    {
        return $this->createService('System:LogService');
    }

    /**
     * @return MemberDao
     */
    protected function getMemberDao()
    {
        return $this->createDao('CorporateTrainingBundle:OfflineCourse:MemberDao');
    }
}
