<?php

namespace CorporateTrainingBundle\Biz\OfflineCourse\Service\Impl;

use Biz\BaseService;
use Biz\Content\Service\FileService;
use Biz\User\Service\UserService;
use CorporateTrainingBundle\Biz\OfflineCourse\Dao\OfflineCourseDao;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\EnrollmentRecordService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\MemberService;
use CorporateTrainingBundle\Biz\OfflineCourse\Service\OfflineCourseService;
use AppBundle\Common\ArrayToolkit;

class OfflineCourseServiceImpl extends BaseService implements OfflineCourseService
{
    public function findOfflineCoursesByIds($ids)
    {
        return $this->getOfflineCourseDao()->findOfflineCoursesByIds($ids);
    }

    public function createOfflineCourse($offlineCourse)
    {
        $this->validateOfflineCourseFields($offlineCourse);
        $offlineCourse = $this->filterOfflineCourseFields($offlineCourse);
        if (!$this->hasCourseCreateRole()) {
            throw $this->createAccessDeniedException('You have no access to Offline Course Management');
        }
        $offlineCourse['status'] = 'draft';
        $offlineCourse['creator'] = $this->getCurrentUser()->getId();
        $offlineCourse = $this->filterNull($offlineCourse);

        return $this->getOfflineCourseDao()->create($offlineCourse);
    }

    public function searchOfflineCourses($conditions, $orderBy, $start, $limit)
    {
        $conditions = $this->prepareSearchConditions($conditions);

        return $this->getOfflineCourseDao()->search($conditions, $orderBy, $start, $limit);
    }

    public function countOfflineCourses($conditions)
    {
        $conditions = $this->prepareSearchConditions($conditions);

        return $this->getOfflineCourseDao()->count($conditions);
    }

    public function getOfflineCourse($id)
    {
        return $this->getOfflineCourseDao()->get($id);
    }

    public function changeCourseCover($id, $coverArray)
    {
        if (empty($coverArray)) {
            throw $this->createInvalidArgumentException('Invalid Param: cover');
        }
        $offlineCourse = $this->getOfflineCourse($id);
        $covers = array();
        foreach ($coverArray as $cover) {
            $file = $this->getFileService()->getFile($cover['id']);
            $covers[$cover['type']] = $file['uri'];
        }

        $offlineCourse = $this->getOfflineCourseDao()->update($offlineCourse['id'], array('cover' => $covers));

        $this->getLogService()->info(
            'offlineCourse',
            'update_picture',
            "更新课程《{$offlineCourse['title']}》(#{$offlineCourse['id']})图片",
            $covers
        );

        return $offlineCourse;
    }

    public function updateOfflineCourse($id, $fields)
    {
        if (!ArrayToolkit::requireds($fields, array('title', 'startDate', 'endDate', 'address', 'maxStudentNum', 'enrollmentEndDate'))) {
            throw $this->createInvalidArgumentException('Lack of required fields');
        }

        $offlineCourse = $this->getOfflineCourse($id);
        $fields = $this->fieldFields($fields);
        $fields = $this->filterNull($fields);

        if (isset($fields['summary'])) {
            $fields['summary'] = $this->purifyHtml($fields['summary'], true);
        }

        $offlineCourse = $this->getOfflineCourseDao()->update($offlineCourse['id'], $fields);

        $this->getLogService()->info('offlineCourse', 'update', "修改课程《{$offlineCourse['title']}》(#{$offlineCourse['id']})");

        return $offlineCourse;
    }

    public function applyAttendOfflineCourse($courseId, $userId)
    {
        if ($this->canApplyAttendOfflineCourse($courseId, $userId)) {
            $record = array(
                'offlineCourseId' => $courseId,
                'userId' => $userId,
                'submittedTime' => time(),
            );

            return $this->getEnrollmentRecordService()->createEnrollmentRecord($record);
        }
    }

    public function publishOfflineCourse($courseId)
    {
        $canManage = $this->hasCoursePublishRole();

        if (!$canManage) {
            throw $this->createServiceException('can not publish offline course');
        }

        return $this->getOfflineCourseDao()->update($courseId, array('status' => 'published'));
    }

    public function closeOfflineCourse($courseId)
    {
        if (!$this->hasCourseCloseRole()) {
            throw $this->createAccessDeniedException('You can not close offline course ');
        }

        $offlineCourse = $this->getOfflineCourseDao()->get($courseId);
        if ($offlineCourse['status'] !== 'published') {
            throw $this->createAccessDeniedException('$offlineCourse has not bean published');
        }

        return $offlineCourse = $this->getOfflineCourseDao()->update($courseId, array('status' => 'closed'));
    }

    public function getUserApplyStatus($courseId, $userId)
    {
        $course = $this->getOfflineCourse($courseId);
        if (time() > $course['enrollmentEndDate']) {
            return 'enrollmentEnd';
        }

        $member = $this->getMemberService()->getMemberByCourseIdAndUserId($courseId, $userId);
        if ('join' === $member['joinStatus']) {
            return 'join';
        }

        $record = $this->getEnrollmentRecordService()->getLatestEnrollmentRecordByCourseIdAndUserId($courseId, $userId);
        if (!empty($record) && 'submitted' == $record['status']) {
            return $record['status'];
        }

        if ($course['studentNum'] < $course['maxStudentNum'] || 0 == $course['maxStudentNum']) {
            return 'enrollAble';
        }

        if ($course['studentNum'] >= $course['maxStudentNum']) {
            return 'enrollUnable';
        }
    }

    public function hasCourseManageRole()
    {
        $user = $this->getCurrentUser();
        if (!$user->isLogin()) {
            return false;
        }

        return $user->hasPermission('admin_offline_course_manage');
    }

    public function hasCourseCreateRole()
    {
        $user = $this->getCurrentUser();
        if (!$user->isLogin()) {
            return false;
        }

        return $user->hasPermission('admin_offline_course_create');
    }

    public function hasCoursePublishRole()
    {
        $user = $this->getCurrentUser();
        if (!$user->isLogin()) {
            return false;
        }

        return $user->hasPermission('admin_offline_course_publish');
    }

    public function hasCourseCloseRole()
    {
        $user = $this->getCurrentUser();
        if (!$user->isLogin()) {
            return false;
        }

        return $user->hasPermission('admin_offline_course_close');
    }

    public function hasCourseSignInRole()
    {
        $user = $this->getCurrentUser();
        if (!$user->isLogin()) {
            return false;
        }

        return $user->hasPermission('admin_offline_course_sign_qrcode');
    }

    public function refreshMemberCount($courseId)
    {
        $conditions = array(
            'courseId' => $courseId,
            'joinStatus' => 'join',
        );

        $count = $this->getMemberService()->countMembers($conditions);

        return $this->getOfflineCourseDao()->update($courseId, array('studentNum' => $count));
    }

    protected function validateOfflineCourseFields($fields)
    {
        if (!ArrayToolkit::requireds($fields, array('title', 'categoryId'))) {
            throw $this->createInvalidArgumentException('parameters is invalid');
        }
        $category = $this->getCategoryService()->getCategory($fields['categoryId']);
        if (empty($category)) {
            throw $this->createInvalidArgumentException('parameters is invalid');
        }
    }

    protected function filterOfflineCourseFields($fields)
    {
        return ArrayToolkit::parts(
            $fields,
            array(
                'title',
                'categoryId',
            )
        );
    }

    protected function canApplyAttendOfflineCourse($courseId, $userId)
    {
        $course = $this->getOfflineCourse($courseId);

        if (empty($course)) {
            return false;
        }

        if ('published' != $course['status']) {
            return false;
        }

        $user = $this->getUserService()->getUser($userId);
        if (empty($user)) {
            return false;
        }

        if (time() > $course['enrollmentEndDate']) {
            return false;
        }

        if (!empty($course['maxStudentNum']) && $course['studentNum'] >= $course['maxStudentNum']) {
            return false;
        }

        $member = $this->getMemberService()->getMemberByCourseIdAndUserId($courseId, $userId);

        if (!empty($member) && 'join' === $member['joinStatus']) {
            return false;
        }

        return true;
    }

    protected function prepareSearchConditions($conditions)
    {
        if (!empty($conditions['searchType']) && 'ongoing' === $conditions['searchType']) {
            $conditions['endDate_GE'] = time();
        }

        if (!empty($conditions['searchType']) && 'end' === $conditions['searchType']) {
            $conditions['endDate_LE'] = time();
        }

        if (!empty($conditions['searchType']) && 'all' === $conditions['searchType']) {
            unset($conditions['searchType']);
        }

        return $conditions;
    }

    /**
     * @return FileService
     */
    protected function getFileService()
    {
        return $this->createService('Content:FileService');
    }

    protected function getLogService()
    {
        return $this->createService('System:LogService');
    }

    protected function fieldFields($fields)
    {
        $fields = ArrayToolkit::parts(
            $fields,
            array(
                'title',
                'categoryId',
                'startDate',
                'endDate',
                'address',
                'maxStudentNum',
                'enrollmentEndDate',
                'summary',
                'requireAudit',
            )
        );

        if (!empty($fields['summary'])) {
            $fields['summary'] = $this->purifyHtml($fields['summary'], true);
        }

        return $fields;
    }

    protected function filterNull($fields)
    {
        return array_filter(
            $fields,
            function ($value) {
                if ($value === '' || $value === null) {
                    return false;
                }

                return true;
            }
        );
    }

    /**
     * @return MemberService
     */
    protected function getMemberService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:MemberService');
    }

    /**
     * @return UserService
     */
    protected function getUserService()
    {
        return $this->createService('User:UserService');
    }

    /**
     * @return EnrollmentRecordService
     */
    protected function getEnrollmentRecordService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:EnrollmentRecordService');
    }

    /**
     * @return OfflineCourseDao
     */
    protected function getOfflineCourseDao()
    {
        return $this->createDao('CorporateTrainingBundle:OfflineCourse:OfflineCourseDao');
    }

    protected function getCategoryService()
    {
        return $this->createService('CorporateTrainingBundle:OfflineCourse:CategoryService');
    }
}
