<?php

namespace CorporateTrainingBundle\Biz\Org\Service\Impl;

use AppBundle\Common\ArrayToolkit;
use CorporateTrainingBundle\Common\Constant\CTConst;
use CorporateTrainingBundle\Biz\Org\Service\OrgService;
use Biz\Org\Service\Impl\OrgServiceImpl as BaseOrgServiceImpl;
use CorporateTrainingBundle\Common\TreeToolkit;
use Codeages\Biz\Framework\Event\Event;

class OrgServiceImpl extends BaseOrgServiceImpl implements OrgService
{
    public function findSelfAndParentOrgsByOrgCode($orgCode)
    {
        $preOrgIdArray = explode('.', $orgCode);

        return $this->findOrgsByIds($preOrgIdArray);
    }

    public function buildOrgTreeByCode($orgCode)
    {
        if (empty($orgCode)) {
            return array();
        }

        $orgInfo = $this->getOrgByOrgCode($orgCode);

        $orgs = $this->findOrgsByPrefixOrgCode($orgCode);

        $orgs = ArrayToolkit::index($orgs, 'id');

        for ($i = CTConst::ORG_MAX_DEPTH; $i >= ($orgInfo['depth'] + 1); --$i) {
            foreach ($orgs as $key => $org) {
                if (!empty($org['depth']) && $org['depth'] == $i && $org['depth'] > 1) {
                    $orgs[$org['parentId']]['nodes'][] = $org;
                    unset($orgs[$key]);
                }
            }
        }

        return array_values($orgs);
    }

    public function resetOrg()
    {
        $currentUser = $this->getCurrentUser();

        if ($currentUser->isSuperAdmin() || $currentUser->hasPermission('admin_org_sync_department')) {
            $org = $this->getOrg(CTConst::ROOT_ORG_CODE);
            $this->getOrgDao()->wave(array($org['id']), array('childrenNum' => -$org['childrenNum']));
            $this->getOrgDao()->update($org['id'], array('syncId' => 0));

            return $this->getOrgDao()->deleteOrgsWithoutOrgCode(CTConst::ROOT_ORG_CODE);
        }

        return false;
    }

    public function createOrg($org)
    {
        $user = $this->getCurrentUser();

        $org = ArrayToolkit::parts($org, array('name', 'code', 'parentId', 'description', 'syncId'));

        if (!ArrayToolkit::requireds($org, array('name', 'code'))) {
            throw $this->createServiceException('缺少必要字段,添加失败');
        }

        $org['createdUserId'] = $user['id'];

        $org = $this->getOrgDao()->create($org);

        $parentOrg = $this->updateParentOrg($org);

        $org = $this->updateOrgCodeAndDepth($org, $parentOrg);

        $this->dispatchEvent('org.create', new Event($org));

        return $org;
    }

    public function updateOrg($id, $fields)
    {
        $org = $this->checkBeforeProccess($id);

        $fields = ArrayToolkit::parts($fields, array('name', 'code', 'parentId', 'description', 'syncId'));

        if (!ArrayToolkit::requireds($fields, array('name', 'code'))) {
            throw $this->createServiceException('缺少必要字段,添加失败');
        }

        $org = $this->getOrgDao()->update($id, $fields);

        $this->dispatchEvent('org.update', new Event($org));

        return $org;
    }

    public function deleteOrg($id)
    {
        $org = $this->checkBeforeProccess($id);

        try {
            $this->biz['db']->beginTransaction();

            if ($org['parentId']) {
                $this->getOrgDao()->wave(array($org['parentId']), array('childrenNum' => -1));
            }

            $this->getOrgDao()->delete($id);
            //删除辖下
            $this->getOrgDao()->deleteByPrefixOrgCode($org['orgCode']);
            $this->dispatchEvent('org.delete', new Event($org));
            $this->biz['db']->commit();
        } catch (\Exception $e) {
            $this->biz['db']->rollback();
            throw $e;
        }
    }

    public function getOrgBySyncId($syncId)
    {
        return $this->getOrgDao()->getOrgBySyncId($syncId);
    }

    public function findOrgsBySyncIds($syncIds)
    {
        return $this->getOrgDao()->findOrgsBySyncIds($syncIds);
    }

    private function updateParentOrg($org)
    {
        $parentOrg = null;

        if (isset($org['parentId']) && $org['parentId'] > 0) {
            $parentOrg = $this->getOrgDao()->get($org['parentId']);
            $this->getOrgDao()->wave(array($parentOrg['id']), array('childrenNum' => +1));
        }

        return $parentOrg;
    }

    private function updateOrgCodeAndDepth($org, $parentOrg)
    {
        $fields = array();

        if (empty($parentOrg)) {
            $fields['orgCode'] = $org['id'].'.';
            $fields['depth'] = 1;
        } else {
            $fields['orgCode'] = $parentOrg['orgCode'].$org['id'].'.';
            $fields['depth'] = $parentOrg['depth'] + 1;
        }

        return $this->getOrgDao()->update($org['id'], $fields);
    }

    private function checkBeforeProccess($id)
    {
        $org = $this->getOrg($id);

        if (empty($org)) {
            throw $this->createServiceException('组织机构不存在,更新失败');
        }

        return $org;
    }

    public function countOrgs(array $conditions)
    {
        return $this->getOrgDao()->count($conditions);
    }

    public function findOrgsByPrefixOrgCodes(array $orgCodes = array('1.'))
    {
        return $this->getOrgDao()->findByPrefixOrgCodes($orgCodes);
    }

    public function buildVisibleOrgTreeByOrgCodes(array $orgCodes)
    {
        return array_values(TreeToolkit::makeTree($this->getVisibleOrgTreeDataByOrgCodes($orgCodes, true), 'seq'));
    }

    public function getVisibleOrgTreeDataByOrgCodes(array $orgCodes, $filter = false)
    {
        $orgs = $this->findOrgsByPrefixOrgCodes($orgCodes);
        $orgCodes = ArrayToolkit::column($orgs, 'orgCode');
        $orgCodeIds = $this->exploreOrgCodes($orgCodes);

        $parentOrgIds = array_diff($orgCodeIds, ArrayToolkit::column($orgs, 'id'));
        $parentOrgs = $this->findOrgsByIds(array_values($parentOrgIds));

        $allOrgs = array_merge($orgs, $parentOrgs);

        if ($filter) {
            $allOrgs = $this->filterOrgs($allOrgs);
        }

        foreach ($allOrgs as &$org) {
            if (in_array($org['id'], $parentOrgIds)) {
                $org['selectable'] = false;
            } else {
                $org['selectable'] = true;
            }
        }

        $indexedAllOrgs = ArrayToolkit::index($allOrgs, 'orgCode');
        ksort($indexedAllOrgs);

        return array_values($indexedAllOrgs);
    }

    public function findOrgsByOrgCodes(array $orgCodes)
    {
        return $this->getOrgDao()->findByOrgCodes($orgCodes);
    }

    public function findOrgsByCodes(array $codes)
    {
        return $this->getOrgDao()->findByCodes($codes);
    }

    protected function filterOrgs($orgs)
    {
        foreach ($orgs as $key => $org) {
            $orgs[$key] = ArrayToolkit::parts($org, array(
                'id',
                'name',
                'parentId',
                'seq',
                'orgCode',
                'code',
            ));
        }

        return $orgs;
    }

    protected function exploreOrgCodes(array $orgCodes)
    {
        $orgIds = array();

        foreach ($orgCodes as $orgCode) {
            $orgIds = array_merge($orgIds, array_filter(explode('.', $orgCode)));
        }

        return array_unique($orgIds);
    }
}
