<?php

namespace CorporateTrainingBundle\Biz\TrainingAssignment\Service\Impl;

use AppBundle\Common\ArrayToolkit;
use Biz\BaseService;
use Biz\Course\Service\CourseService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Dao\TrainingAssignmentItemDao;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentItemService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentService;

class TrainingAssignmentItemServiceImpl extends BaseService implements TrainingAssignmentItemService
{
    public function getTrainingAssignmentItem($id)
    {
        return $this->getTrainingAssignmentItemDao()->get($id);
    }

    public function findTrainingAssignmentItemsByTargetIdAndTargetType($targetId, $targetType)
    {
        return $this->getTrainingAssignmentItemDao()->findByTargetIdAndTargetType($targetId, $targetType);
    }

    public function findTrainingAssignmentItemsByAssignmentId($assignmentId)
    {
        return $this->getTrainingAssignmentItemDao()->findByAssignmentId($assignmentId);
    }

    public function findTrainingAssignmentItemsByAssignmentIds(array $assignmentIds)
    {
        return $this->getTrainingAssignmentItemDao()->findByAssignmentIds($assignmentIds);
    }

    public function createTrainingAssignmentItem($item)
    {
        $fields = $this->filterTrainingAssignmentItemFields($item);

        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($fields['assignmentId'])) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        if (!ArrayToolkit::requireds($fields, array('assignmentId', 'targetType', 'targetId'))) {
            throw $this->createInvalidArgumentException('Lack of required fields');
        }

        if (!in_array($fields['targetType'], array('course', 'classroom'))) {
            throw $this->createInvalidArgumentException('Param Invalid: TargetType');
        }

        return $this->getTrainingAssignmentItemDao()->create($fields);
    }

    public function updateTrainingAssignmentItem($id, $fields)
    {
        $item = $this->getTrainingAssignmentItem($id);

        if (empty($item)) {
            $this->createNotFoundException("Assignment#{$id} Not Found");
        }

        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($item['assignmentId'])) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        $fields = $this->filterTrainingAssignmentItemFields($fields);

        return $this->getTrainingAssignmentItemDao()->update($id, $fields);
    }

    public function countTrainingAssignmentItems(array $conditions)
    {
        return $this->getTrainingAssignmentItemDao()->count($conditions);
    }

    public function searchTrainingAssignmentItems(array $conditions, array $orderBys, $start, $limit)
    {
        return $this->getTrainingAssignmentItemDao()->search($conditions, $orderBys, $start, $limit);
    }

    public function deleteTrainingAssignmentItem($id)
    {
        $item = $this->getTrainingAssignmentItem($id);

        if (empty($item)) {
            $this->createNotFoundException("Assignment#{$id} Not Found");
        }

        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($item['assignmentId'])) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        return $this->getTrainingAssignmentItemDao()->delete($id);
    }

    public function deleteItemByAssignmentId($assignmentId)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($assignmentId)) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        return $this->getTrainingAssignmentItemDao()->deleteItemByAssignmentId($assignmentId);
    }

    public function findAssignmentItemsByAssignmentIdAndTargetType($assignmentId, $targetType)
    {
        return $this->getTrainingAssignmentItemDao()->findByAssignmentIdAndTargetType($assignmentId, $targetType);
    }

    public function setAssignmentCourseItems($assignmentId, $courseItems)
    {
        $assignment = $this->getTrainingAssignmentService()->getTrainingAssignment($assignmentId);

        if (empty($assignment)) {
            throw  $this->createNotFoundException('Assignment Item Not Found');
        }

        $assignmentItems = array();
        foreach ($courseItems as $key => $courseItem) {
            if (empty($courseItem)) {
                throw $this->createNotFoundException('Course Id Not Found');
            }

            $course = $this->getCourseService()->getCourse($courseItem);

            if (empty($course)) {
                throw $this->createNotFoundException('Course Not Found');
            }

            $assignmentItems[] = array(
                'targetId' => $courseItem,
                'assignmentId' => $assignmentId,
                'seq' => $key,
                'targetType' => 'course',
            );
        }

        $existCourseItems = $this->findAssignmentItemsByAssignmentIdAndTargetType($assignmentId, 'course');

        foreach ($existCourseItems as $existCourseItem) {
            $this->getTrainingAssignmentItemDao()->delete($existCourseItem['id']);
        }

        foreach ($assignmentItems as $assignmentItem) {
            $this->getTrainingAssignmentItemDao()->create($assignmentItem);
        }
    }

    protected function filterTrainingAssignmentItemFields($fields)
    {
        return ArrayToolkit::parts(
            $fields,
            array(
                'assignmentId',
                'targetType',
                'targetId',
                'seq',
            )
        );
    }

    /**
     * @return TrainingAssignmentService
     */
    protected function getTrainingAssignmentService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentService');
    }

    /**
     * @return TrainingAssignmentItemDao
     */
    protected function getTrainingAssignmentItemDao()
    {
        return $this->createDao('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentItemDao');
    }

    /**
     * @return CourseService
     */
    protected function getCourseService()
    {
        return $this->createService('Course:CourseService');
    }
}
