<?php

namespace CorporateTrainingBundle\Biz\TrainingAssignment\Service\Impl;

use AppBundle\Common\ArrayToolkit;
use Biz\BaseService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentMemberService;

class TrainingAssignmentMemberServiceImpl extends BaseService implements TrainingAssignmentMemberService
{
    public function getTrainingAssignmentMember($id)
    {
        return $this->getTrainingAssignmentMemberDao()->get($id);
    }

    public function getTrainingAssignmentMemberByUserIdAndAssignmentId($userId, $assignmentId)
    {
        return $this->getTrainingAssignmentMemberDao()->getByUserIdAndAssignmentId($userId, $assignmentId);
    }

    public function findMembersByUserId($userId)
    {
        return $this->getTrainingAssignmentMemberDao()->findByUserId($userId);
    }

    public function findMembersByAssignmentId($assignmentId)
    {
        return $this->getTrainingAssignmentMemberDao()->findByAssignmentId($assignmentId);
    }

    public function isMemberExit($userId, $assignmentId)
    {
        $member = $this->getTrainingAssignmentMemberByUserIdAndAssignmentId($userId, $assignmentId);

        return empty($member) ? false : true;
    }

    public function createTrainingAssignmentMember($member)
    {
        $fields = $this->filterTrainingAssignmentMemberFields($member);

        if (!ArrayToolkit::requireds($fields, array('assignmentId', 'userId'))) {
            throw $this->createInvalidArgumentException('Lack of required fields');
        }

        $currentUser = $this->getCurrentUser();
        $fields['operatedUserId'] = $currentUser['id'];

        return $this->getTrainingAssignmentMemberDao()->create($fields);
    }

    public function batchBecomeMember($assignmentId, $userIds)
    {
        if (empty($userIds)) {
            return;
        }

        $assignment = $this->getTrainingAssignmentService()->getTrainingAssignment($assignmentId);

        if (empty($assignment)) {
            throw $this->createNotFoundException();
        }

        if (!in_array($assignment['status'], array('published'))) {
            throw $this->createServiceException('不能加入未发布的培训项目');
        }

        $users = $this->getUserService()->searchUsers(
            array('userIds' => $userIds),
            array(),
            0,
            PHP_INT_MAX
        );
        $userIds = ArrayToolkit::column($users, 'id');

        foreach ($userIds as $userId) {
            $user = $this->getUserService()->getUser($userId);
            if (empty($user)) {
                throw $this->createServiceException("用户(#{$userId})不存在，加入培训项目失败！");
            }

            $member = $this->getTrainingAssignmentMemberByUserIdAndAssignmentId($userId, $assignmentId);
            if (empty($member)) {
                $fields = array(
                    'assignmentId' => $assignmentId,
                    'userId' => $userId,
                );
                $progress = $this->createAssignmentStrategy()->calculateMemberLearnProgress($assignment, $userId);
                if (100 == $progress) {
                    $fields['status'] = 'finished';
                    $fields['finishedTime'] = time();
                }
                $this->createTrainingAssignmentMember($fields);
            }
        }

        return true;
    }

    public function updateTrainingAssignmentMember($id, $fields)
    {
        $fields = $this->filterTrainingAssignmentMemberFields($fields);

        return $this->getTrainingAssignmentMemberDao()->update($id, $fields);
    }

    public function countTrainingAssignmentMembers(array $conditions)
    {
        return $this->getTrainingAssignmentMemberDao()->count($conditions);
    }

    public function searchTrainingAssignmentMembers(array $conditions, array $orderBys, $start, $limit)
    {
        return $this->getTrainingAssignmentMemberDao()->search($conditions, $orderBys, $start, $limit);
    }

    public function deleteTrainingAssignmentMember($id)
    {
        $member = $this->getTrainingAssignmentMember($id);

        if (empty($member)) {
            throw $this->createNotFoundException('Assignment Member Not Found');
        }

        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($member['assignmentId'])) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        return $this->getTrainingAssignmentMemberDao()->delete($id);
    }

    public function deleteMemberByAssignmentId($assignmentId)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($assignmentId)) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        return $this->getTrainingAssignmentMemberDao()->deleteMemberByAssignmentId($assignmentId);
    }

    public function isBelongToUserAssignment($userId, $itemId, $type)
    {
        if (empty($userId) || empty($itemId) || empty($type)) {
            return false;
        }

        if (!in_array($type, array('course'))) {
            return false;
        }

        $members = $this->findMembersByUserId($userId);
        if (empty($members)) {
            return false;
        }

        $assignmentItems = $this->getTrainingAssignmentItemService()->searchTrainingAssignmentItems(
            array('targetType' => $type, 'targetId' => $itemId),
            array(),
            0,
            PHP_INT_MAX
        );

        if (empty($assignmentItems)) {
            return false;
        }

        $trainingAssignmentIds = ArrayToolkit::column($assignmentItems, 'assignmentId');

        $publishedAssignment = $this->getTrainingAssignmentService()->searchTrainingAssignments(
            array('ids' => $trainingAssignmentIds, 'status' => 'published'),
            array(),
            0,
            PHP_INT_MAX
        );

        $publishedAssignmentIds = ArrayToolkit::column($publishedAssignment, 'id');

        $members = $this->searchTrainingAssignmentMembers(
            array('assignmentIds' => $publishedAssignmentIds, 'userId' => $userId),
            array(),
            0,
            PHP_INT_MAX
        );

        return empty($members) ? false : true;
    }

    public function batchDeleteMembers($assignmentId, $memberIds)
    {
        if (!$this->getTrainingAssignmentService()->canManageTrainingAssignment($assignmentId)) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        foreach ($memberIds as $memberId) {
            $this->getTrainingAssignmentMemberDao()->delete($memberId);
        }
    }

    protected function filterTrainingAssignmentMemberFields($fields)
    {
        return ArrayToolkit::parts($fields, array('assignmentId', 'userId', 'status', 'finishedTime'));
    }

    protected function createAssignmentStrategy()
    {
        return $this->biz->offsetGet('assignment_strategy_context')->createStrategy();
    }

    /**
     * @return \CorporateTrainingBundle\Biz\TrainingAssignment\Service\Impl\TrainingAssignmentServiceImpl
     */
    protected function getTrainingAssignmentService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentService');
    }

    /**
     * @return \CorporateTrainingBundle\Biz\TrainingAssignment\Service\Impl\TrainingAssignmentItemServiceImpl
     */
    protected function getTrainingAssignmentItemService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentItemService');
    }

    /**
     * @return \CorporateTrainingBundle\Biz\TrainingAssignment\Service\Impl\TrainingAssignmentMemberServiceImpl
     */
    protected function getTrainingAssignmentMemberDao()
    {
        return $this->createDao('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentMemberDao');
    }

    /**
     * @return \Biz\User\Service\Impl\UserServiceImpl
     */
    protected function getUserService()
    {
        return $this->createService('User:UserService');
    }
}
