<?php

namespace CorporateTrainingBundle\Biz\TrainingAssignment\Service\Impl;

use AppBundle\Common\ArrayToolkit;
use Biz\BaseService;
use Biz\Content\Service\FileService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Dao\TrainingAssignmentDao;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentItemService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentMemberService;
use CorporateTrainingBundle\Biz\TrainingAssignment\Service\TrainingAssignmentService;

class TrainingAssignmentServiceImpl extends BaseService implements TrainingAssignmentService
{
    public function getTrainingAssignment($id)
    {
        return $this->getTrainingAssignmentDao()->get($id);
    }

    public function createTrainingAssignment($trainingAssignment)
    {
        if (!$this->hasManageTrainingAssignmentRole()) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        $fields = $this->filterTrainingAssignmentFields($trainingAssignment);

        if (!ArrayToolkit::requireds($fields, array('name', 'orgId', 'orgCode'))) {
            throw $this->createInvalidArgumentException('Lack of required fields');
        }

        $currentUser = $this->getCurrentUser();
        $fields['createdUserId'] = $currentUser['id'];

        return $this->getTrainingAssignmentDao()->create($fields);
    }

    public function updateTrainingAssignment($id, $trainingAssignment)
    {
        if (!$this->canManageTrainingAssignment($id)) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        $fields = $this->filterTrainingAssignmentFields($trainingAssignment);

        return $this->getTrainingAssignmentDao()->update($id, $fields);
    }

    public function changeCover($id, $coverArray)
    {
        if (empty($coverArray)) {
            throw $this->createInvalidArgumentException('Invalid Param: cover');
        }
        $assignment = $this->getTrainingAssignment($id);
        $covers = array();
        foreach ($coverArray as $cover) {
            $file = $this->getFileService()->getFile($cover['id']);
            $covers[$cover['type']] = $file['uri'];
        }

        return $this->getTrainingAssignmentDao()->update($assignment['id'], array('cover' => $covers));
    }

    public function countTrainingAssignments(array $conditions)
    {
        return $this->getTrainingAssignmentDao()->count($conditions);
    }

    public function searchTrainingAssignments(array $conditions, array $orderBys, $start, $limit)
    {
        return $this->getTrainingAssignmentDao()->search($conditions, $orderBys, $start, $limit);
    }

    public function deleteTrainingAssignmentById($id)
    {
        if (!$this->canManageTrainingAssignment($id)) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        $trainingAssignment = $this->getTrainingAssignment($id);
        if (empty($trainingAssignment)) {
            throw $this->createNotFoundException('Training Assignment Not Found');
        }

        if ($trainingAssignment['status'] == 'published') {
            throw $this->createAccessDeniedException('Delete Published Training Assignment Is Not Allowed');
        }

        try {
            $this->beginTransaction();

            $this->getTrainingAssignmentItemService()->deleteItemByAssignmentId($id);
            $this->getTrainingAssignmentMemberService()->deleteMemberByAssignmentId($id);
            $result = $this->getTrainingAssignmentDao()->delete($id);

            $this->commit();

            return $result;
        } catch (\Exception $e) {
            $this->rollback();
            throw $e;
        }
    }

    public function publishTrainingAssignment($id)
    {
        if (!$this->canManageTrainingAssignment($id)) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        $trainingAssignment = $this->getTrainingAssignment($id);
        if (empty($trainingAssignment)) {
            throw $this->createNotFoundException('Training Assignment Not Found');
        }

        return $this->updateTrainingAssignment($id, array('status' => 'published'));
    }

    public function closeTrainingAssignment($id)
    {
        if (!$this->canManageTrainingAssignment($id)) {
            throw $this->createAccessDeniedException('Access Denied');
        }

        $trainingAssignment = $this->getTrainingAssignment($id);
        if (empty($trainingAssignment)) {
            throw $this->createNotFoundException('Training Assignment Not Found');
        }

        return $this->updateTrainingAssignment($id, array('status' => 'closed'));
    }

    protected function filterTrainingAssignmentFields($fields)
    {
        return ArrayToolkit::parts(
            $fields,
            array(
                'name',
                'startTime',
                'endTime',
                'status',
                'summary',
                'createdUserId',
                'orgId',
                'orgCode',
                'cover',
                'itemNum',
                'memberNum',
            )
        );
    }

    public function canManageTrainingAssignment($id)
    {
        $currentUser = $this->getCurrentUser();
        if (!$currentUser->isLogin()) {
            return false;
        }

        $trainingAssignment = $this->getTrainingAssignment($id);
        if (empty($trainingAssignment)) {
            return false;
        }

        if (!in_array($trainingAssignment['orgId'], $currentUser['accessibleOrgIds'])) {
            return false;
        }

        return $this->hasManageTrainingAssignmentRole();
    }

    public function hasManageTrainingAssignmentRole()
    {
        $currentUser = $this->getCurrentUser();
        if (!$currentUser->isLogin()) {
            return false;
        }

        return in_array('ROLE_TRAINING_ADMIN', $currentUser['roles']) || in_array(
                'ROLE_SUPER_ADMIN',
                $currentUser['roles']
            );
    }

    /**
     * @return TrainingAssignmentDao
     */
    protected function getTrainingAssignmentDao()
    {
        return $this->createDao('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentDao');
    }

    /**
     * @return TrainingAssignmentMemberService
     */
    protected function getTrainingAssignmentMemberService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentMemberService');
    }

    /**
     * @return FileService
     */
    protected function getFileService()
    {
        return $this->createService('Content:FileService');
    }

    /**
     * @return TrainingAssignmentItemService
     */
    protected function getTrainingAssignmentItemService()
    {
        return $this->createService('CorporateTrainingBundle:TrainingAssignment:TrainingAssignmentItemService');
    }
}
