<?php

namespace CorporateTrainingBundle\Biz\User\Service\Impl;

use Codeages\Biz\Framework\Util\ArrayToolkit;
use CorporateTrainingBundle\Biz\User\Service\UserService;
use Biz\User\Service\Impl\UserServiceImpl as BaseServiceImpl;
use Codeages\Biz\Framework\Event\Event;
use AppBundle\Common\SimpleValidator;
use CorporateTrainingBundle\Common\Constant\CTConst;

class UserServiceImpl extends BaseServiceImpl implements UserService
{
    public function searchUsers(array $conditions, array $orderBy, $start, $limit)
    {
        if (isset($conditions['nickname'])) {
            $conditions['nickname'] = strtoupper($conditions['nickname']);
        }

        $users = $this->getUserDao()->searchUsers($conditions, $orderBy, $start, $limit);

        return UserSerialize::unserializes($users);
    }

    public function countUsers(array $conditions)
    {
        if (isset($conditions['nickname'])) {
            $conditions['nickname'] = strtoupper($conditions['nickname']);
        }

        return $this->getUserDao()->countUsers($conditions);
    }

    public function unlockUser($id)
    {
        if ($this->isOverMaxUsersNumber()) {
            throw $this->createServiceException('激活用户数达到上限，无法继续解禁用户。如需提升上限请联系您的客户代表');
        }
        parent::unlockUser($id);
    }

    public function initPassword($id, $newPassword)
    {
        $connection = $this->getKernel()->getConnection();
        $connection->beginTransaction();

        try {
            $init = array(
                'pwdInit' => '1',
            );

            $this->getAuthService()->changePassword($id, null, $newPassword);
            $this->getUserDao()->update($id, $init);

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }

        return $this->getUserDao()->update($id, $init);
    }

    public function changePwdInit($id)
    {
        $init = array(
            'pwdInit' => '1',
        );

        return $this->getUserDao()->update($id, $init);
    }

    public function readGuide($id)
    {
        $user = $this->getUser($id);

        if (empty($user)) {
            throw $this->createNotFoundException('User', $id, '用户不存在');
        }

        $updatedUser = $this->getUserDao()->update($id, array('readGuide' => 1));

        return $updatedUser;
    }

    public function changeUserPost($id, $postId)
    {
        $user = $this->getUser($id);

        if (empty($user)) {
            throw $this->createNotFoundException('User', $id, '用户不存在');
        }

        $post = $this->getPostService()->getPost($postId);

        if (empty($post)) {
            throw $this->createNotFoundException('Post', $id, '岗位不存在');
        }

        $updatedUser = $this->getUserDao()->update($id, array('postId' => $postId));
        $this->dispatchEvent('user.post.update', new Event($updatedUser));
        $this->dispatchEvent('user.change_post', new Event($updatedUser));
        $this->getLogService()->info('user', 'post_change', "修改用户{$user['nickname']}岗位为{$post['name']}成功");

        return $updatedUser;
    }

    public function statisticsOrgUserNumGroupByOrgId()
    {
        return $this->getUserDao()->statisticsOrgUserNumGroupByOrgId();
    }

    public function statisticsPostUserNumGroupByPostId()
    {
        return $this->getUserDao()->statisticsPostUserNumGroupByPostId();
    }

    public function batchUpdatePost($ids, $postId)
    {
        $ids = explode(',', $ids);
        $connection = $this->getKernel()->getConnection();
        $connection->beginTransaction();

        try {
            foreach ($ids as $id) {
                $this->changeUserPost($id, $postId);
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function countUsersByLockedStatus()
    {
        $result = array(
            'enable' => $this->countUsers(array('locked' => 0, 'noType' => 'system')),
            'disable' => $this->countUsers(array('locked' => 1)),
        );

        return $result;
    }

    public function isOverMaxUsersNumber()
    {
        $counts = $this->countUsersByLockedStatus();
        $maxUsersNumber = $this->getMaxUsersNumber();

        return ($maxUsersNumber > 0) && ($counts['enable'] >= $maxUsersNumber);
    }

    public function getMaxUsersNumber()
    {
        $maxUsersNumber = $this->getSettingService()->get('max_users_number');

        return empty($maxUsersNumber) ? 0 : $maxUsersNumber;
    }

    public function changeUserOrgs($userId, $orgCodes)
    {
        $user = $this->getUser($userId);
        if (empty($user) || ($user['orgCodes'] == $orgCodes)) {
            return array();
        }

        $orgCodes = $this->filterOrgCodes($orgCodes);
        $orgs = $this->getOrgService()->findOrgsByOrgCodes($orgCodes);
        $existOrgCodes = ArrayToolkit::column($orgs, 'orgCode');

        if (array_diff($orgCodes, $existOrgCodes)) {
            throw $this->createNotFoundException('Org Not Found');
        }
        $this->getKernel()->getConnection()->beginTransaction();
        try {
            $fields = array(
                'orgCodes' => empty($orgCodes) ? array(1.) : ArrayToolkit::column($orgs, 'orgCode'),
                'orgIds' => empty($orgCodes) ? array(1) : ArrayToolkit::column($orgs, 'id'),
            );

            $this->getUserDao()->update($userId, $fields);
            $this->getUserOrgService()->setUserOrgs($userId, $orgs);
            $this->getKernel()->getConnection()->commit();
        } catch (\Exception $e) {
            $this->getKernel()->getConnection()->rollBack();
            throw $e;
        }

        return $user;
    }

    public function batchUpdateOrg($userIds, $orgCode)
    {
        if (!is_array($userIds)) {
            $userIds = array($userIds);
        }
        $fields = $this->fillOrgId(array('orgCode' => $orgCode));

        foreach ($userIds as $userId) {
            $user = $this->getUserDao()->update($userId, $fields);
            $this->dispatchEvent('user.org.update', new Event($user));
        }
    }

    public function getUserBindByTypeAndFromId($type, $fromId)
    {
        if ($type == 'weixinweb' || $type == 'weixinmob') {
            $type = 'weixin';
        }

        if ($type == 'dingtalkweb' || $type == 'dingtalkmob') {
            $type = 'dingtalk';
        }

        return $this->getUserBindDao()->getByTypeAndFromId($type, $fromId);
    }

    public function getUserBindByTypeAndUserId($type, $toId)
    {
        $user = $this->getUserDao()->get($toId);

        if (empty($user)) {
            throw $this->createNotFoundException("User#{$toId} Not Found");
        }

        if (!$this->typeInOAuthClient($type)) {
            throw $this->createInvalidArgumentException('Invalid Type');
        }

        if ($type == 'weixinweb' || $type == 'weixinmob') {
            $type = 'weixin';
        }

        if ($type == 'dingtalkweb' || $type == 'dingtalkmob') {
            $type = 'dingtalk';
        }

        return $this->getUserBindDao()->getByToIdAndType($type, $toId);
    }

    public function bindUser($type, $fromId, $toId, $token)
    {
        $user = $this->getUserDao()->get($toId);

        if (empty($user)) {
            throw $this->createNotFoundException("User#{$toId} Not Found");
        }

        if (!$this->typeInOAuthClient($type)) {
            throw $this->createInvalidArgumentException('Invalid Type');
        }

        if ($type == 'weixinweb' || $type == 'weixinmob') {
            $type = 'weixin';
        }

        if ($type == 'dingtalkweb' || $type == 'dingtalkmob') {
            $type = 'dingtalk';
        }

        $bind = $this->getUserBindDao()->create(array(
            'type' => $type,
            'fromId' => $fromId,
            'toId' => $toId,
            'token' => empty($token['token']) ? '' : $token['token'],
            'createdTime' => time(),
            'expiredTime' => empty($token['expiredTime']) ? 0 : $token['expiredTime'],
        ));

        $this->dispatchEvent('user.bind', new Event($bind));
    }

    public function register($registration, $type = 'default')
    {
        $this->validateNickname($registration['nickname']);

        if (!$this->isNicknameAvaliable($registration['nickname'])) {
            throw $this->createInvalidArgumentException('Nickname Occupied');
        }

        if (!SimpleValidator::email($registration['email'])) {
            throw $this->createInvalidArgumentException('Invalid Email');
        }

        if (!$this->isEmailAvaliable($registration['email'])) {
            throw $this->createInvalidArgumentException('Email Occupied');
        }

        $user = array();

        if (isset($registration['verifiedMobile'])) {
            $user['verifiedMobile'] = $registration['verifiedMobile'];
        } else {
            $user['verifiedMobile'] = '';
        }

        $user['email'] = $registration['email'];
        $user['emailVerified'] = isset($registration['emailVerified']) ? $registration['emailVerified'] : 0;
        $user['nickname'] = $registration['nickname'];
        $user['roles'] = array('ROLE_USER');
        $user['type'] = isset($registration['type']) ? $registration['type'] : $type;
        $user['createdIp'] = empty($registration['createdIp']) ? '' : $registration['createdIp'];
        if (isset($registration['guid'])) {
            $user['guid'] = $registration['guid'];
        }

        $user['createdTime'] = time();
        $user['registeredWay'] = isset($registration['registeredWay']) ? $registration['registeredWay'] : '';

        $thirdLoginInfo = $this->getSettingService()->get('login_bind', array());

        if (in_array($type, array('default', 'phpwind', 'discuz'))) {
            $user['salt'] = base_convert(sha1(uniqid(mt_rand(), true)), 16, 36);
            $user['password'] = $this->getPasswordEncoder()->encodePassword($registration['password'], $user['salt']);
            $user['setup'] = 1;
        } elseif (in_array(
                $type,
                array('qq', 'weibo', 'renren', 'weixinweb', 'weixinmob', 'dingtalkweb', 'dingtalkmob')
            ) && isset($thirdLoginInfo["{$type}_set_fill_account"]) && $thirdLoginInfo["{$type}_set_fill_account"]
        ) {
            $user['salt'] = '';
            $user['password'] = '';
            $user['setup'] = 1;
        } else {
            $user['salt'] = '';
            $user['password'] = '';
            $user['setup'] = 0;
        }
        if (isset($registration['orgIds'])) {
            $user['orgIds'] = $registration['orgIds'];
            $user['orgCodes'] = $registration['orgCodes'];
        }
        $user = $this->getUserDao()->create($user);

        if (!empty($registration['invite_code'])) {
            $inviteUser = $this->getUserDao()->getByInviteCode($registration['invite_code']);
        }

        if (!empty($inviteUser)) {
            $this->getInviteRecordService()->createInviteRecord($inviteUser['id'], $user['id']);
            $invitedCoupon = $this->getCouponService()->generateInviteCoupon($user['id'], 'register');

            if (!empty($invitedCoupon)) {
                $card = $this->getCardService()->getCardByCardId($invitedCoupon['id']);
                $this->getInviteRecordService()->addInviteRewardRecordToInvitedUser(
                    $user['id'],
                    array('invitedUserCardId' => $card['cardId'])
                );
            }

            $this->dispatchEvent(
                'user.register',
                new Event(array('userId' => $user['id'], 'inviteUserId' => $inviteUser['id']))
            );
        }

        if (isset($registration['mobile']) && $registration['mobile'] != '' && !SimpleValidator::mobile(
                $registration['mobile']
            )
        ) {
            throw $this->createInvalidArgumentException('Invalid Mobile');
        }

        if (isset($registration['idcard']) && $registration['idcard'] != '' && !SimpleValidator::idcard(
                $registration['idcard']
            )
        ) {
            throw $this->createInvalidArgumentException('Invalid ID number');
        }

        if (isset($registration['truename']) && $registration['truename'] != '' && !SimpleValidator::truename(
                $registration['truename']
            )
        ) {
            throw $this->createInvalidArgumentException('Invalid truename');
        }

        $profile = array();
        $profile['id'] = $user['id'];
        $profile['mobile'] = empty($registration['mobile']) ? '' : $registration['mobile'];
        $profile['idcard'] = empty($registration['idcard']) ? '' : $registration['idcard'];
        $profile['truename'] = empty($registration['truename']) ? '' : $registration['truename'];
        $profile['company'] = empty($registration['company']) ? '' : $registration['company'];
        $profile['job'] = empty($registration['job']) ? '' : $registration['job'];
        $profile['weixin'] = empty($registration['weixin']) ? '' : $registration['weixin'];
        $profile['weibo'] = empty($registration['weibo']) ? '' : $registration['weibo'];
        $profile['qq'] = empty($registration['qq']) ? '' : $registration['qq'];
        $profile['site'] = empty($registration['site']) ? '' : $registration['site'];
        $profile['gender'] = empty($registration['gender']) ? 'secret' : $registration['gender'];

        for ($i = 1; $i <= 5; ++$i) {
            $profile['intField'.$i] = empty($registration['intField'.$i]) ? null : $registration['intField'.$i];
            $profile['dateField'.$i] = empty($registration['dateField'.$i]) ? null : $registration['dateField'.$i];
            $profile['floatField'.$i] = empty($registration['floatField'.$i]) ? null : $registration['floatField'.$i];
        }

        for ($i = 1; $i <= 10; ++$i) {
            $profile['varcharField'.$i] = empty($registration['varcharField'.$i]) ? '' : $registration['varcharField'.$i];
            $profile['textField'.$i] = empty($registration['textField'.$i]) ? '' : $registration['textField'.$i];
        }

        $this->getProfileDao()->create($profile);

        if ($type != 'default') {
            $this->bindUser($type, $registration['token']['userId'], $user['id'], $registration['token']);
        }

        $this->dispatchEvent('user.registered', new Event($user));

        return $user;
    }

    public function batchUpdateOrgs($userIds, $orgCodes)
    {
        $currentUser = $this->getCurrentUser();
        $userIds = $this->filterUserIds($userIds);
        foreach ($userIds as $userId) {
            if ($currentUser['id'] == $userId) {
                continue;
            }

            $this->changeUserOrgs($userId, $orgCodes);
        }
    }

    protected function filterOrgCodes($orgCodes)
    {
        if (is_array($orgCodes)) {
            return $orgCodes;
        } else {
            return explode('|', $orgCodes);
        }
    }

    protected function filterUserIds($userIds)
    {
        if (is_array($userIds)) {
            return $userIds;
        } else {
            return explode(',', $userIds);
        }
    }

    public function initOrgsRelation()
    {
        $fields = array('|1|', '|1.|');

        $org = $this->getOrgService()->getOrgByOrgCode(CTConst::ROOT_ORG_CODE);
        $users = $this->searchUsers(array(), array(), 0, PHP_INT_MAX);
        if (!empty($users)) {
            foreach ($users as $user) {
                $this->getUserOrgService()->setUserOrgs($user['id'], array($org));
            }
        }

        return $this->getUserDao()->initOrgsRelation($fields);
    }

    public function initSystemUsers()
    {
        $users = array(
            array(
                'type' => 'system',
                'roles' => array('ROLE_USER', 'ROLE_SUPER_ADMIN'),
            ),
        );
        foreach ($users as $user) {
            $existsUser = $this->getUserDao()->getUserByType($user['type']);

            if (!empty($existsUser)) {
                continue;
            }

            $user['nickname'] = $this->generateNickname($user).'(系统用户)';
            $user['emailVerified'] = 1;
            $user['password'] = $this->getRandomChar();
            $user['email'] = $this->generateEmail($user);
            $user['salt'] = base_convert(sha1(uniqid(mt_rand(), true)), 16, 36);
            $user['password'] = $this->getPasswordEncoder()->encodePassword($user['password'], $user['salt']);
            $user = UserSerialize::unserialize(
                $this->getUserDao()->create(UserSerialize::serialize($user))
            );

            $profile = array();
            $profile['id'] = $user['id'];
            $this->getProfileDao()->create($profile);
        }
    }

    protected function getAuthService()
    {
        return $this->createService('User:AuthService');
    }

    protected function getPostService()
    {
        return $this->createService('CorporateTrainingBundle:Post:PostService');
    }

    /**
     * @return \CorporateTrainingBundle\Biz\User\Service\UserOrgService
     */
    protected function getUserOrgService()
    {
        return $this->createService('User:UserOrgService');
    }
}

class UserSerialize
{
    public static function serialize(array $user)
    {
        return $user;
    }

    public static function unserialize(array $user = null)
    {
        if (empty($user)) {
            return;
        }

        $user = self::_userRolesSort($user);

        return $user;
    }

    public static function unserializes(array $users)
    {
        return array_map(
            function ($user) {
                return UserSerialize::unserialize($user);
            },
            $users
        );
    }

    private static function _userRolesSort($user)
    {
        if (!empty($user['roles'][1]) && $user['roles'][1] == 'ROLE_USER') {
            $temp = $user['roles'][1];
            $user['roles'][1] = $user['roles'][0];
            $user['roles'][0] = $temp;
        }

        //交换学员角色跟roles数组第0个的位置;

        return $user;
    }
}
